@@ -1,6 +1,6 @@
 #include "ruby.h"
+#include "internal/time.h"
 
-void ruby_reset_leap_second_info(void);
 static VALUE
 bug_time_s_reset_leap_second_info(VALUE klass)
 {
