@@ -5,13 +5,15 @@
  * Documented by mathew <meta@pobox.com>
  *
  * $RoughId: syslog.c,v 1.21 2002/02/25 12:21:17 knu Exp $
- * $Id: syslog.c 60071 2017-09-30 08:35:23Z nobu $
+ * $Id$
  */
 
 #include "ruby/ruby.h"
 #include "ruby/util.h"
 #include <syslog.h>
 
+#define SYSLOG_VERSION "0.1.2"
+
 /* Syslog class */
 static VALUE mSyslog;
 /*
@@ -162,19 +164,18 @@ static VALUE mSyslog_open(int argc, VALUE *argv, VALUE self)
         ident = rb_gv_get("$0");
     }
     ident_ptr = StringValueCStr(ident);
-    rb_check_safe_obj(ident);
     syslog_ident = strdup(ident_ptr);
 
     if (NIL_P(opt)) {
-	syslog_options = LOG_PID | LOG_CONS;
+        syslog_options = LOG_PID | LOG_CONS;
     } else {
-	syslog_options = NUM2INT(opt);
+        syslog_options = NUM2INT(opt);
     }
 
     if (NIL_P(fac)) {
-	syslog_facility = LOG_USER;
+        syslog_facility = LOG_USER;
     } else {
-	syslog_facility = NUM2INT(fac);
+        syslog_facility = NUM2INT(fac);
     }
 
     openlog(syslog_ident, syslog_options, syslog_facility);
@@ -297,10 +298,6 @@ static VALUE mSyslog_set_mask(VALUE self, VALUE mask)
  *   Syslog.log(Syslog::LOG_ALERT, "Out of memory")
  *   Syslog.alert("Out of memory")
  *
- * Format strings are as for printf/sprintf, except that in addition %m is
- * replaced with the error message string that would be returned by
- * strerror(errno).
- *
  */
 static VALUE mSyslog_log(int argc, VALUE *argv, VALUE self)
 {
@@ -312,7 +309,7 @@ static VALUE mSyslog_log(int argc, VALUE *argv, VALUE self)
     pri = *argv++;
 
     if (!FIXNUM_P(pri)) {
-	rb_raise(rb_eTypeError, "type mismatch: %"PRIsVALUE" given", rb_obj_class(pri));
+        rb_raise(rb_eTypeError, "type mismatch: %"PRIsVALUE" given", rb_obj_class(pri));
     }
 
     syslog_write(FIX2INT(pri), argc, argv);
@@ -327,14 +324,14 @@ static VALUE mSyslog_inspect(VALUE self)
     Check_Type(self, T_MODULE);
 
     if (!syslog_opened)
-	return rb_sprintf("<#%"PRIsVALUE": opened=false>", self);
+        return rb_sprintf("<#%"PRIsVALUE": opened=false>", self);
 
     return rb_sprintf("<#%"PRIsVALUE": opened=true, ident=\"%s\", options=%d, facility=%d, mask=%d>",
-		      self,
-		      syslog_ident,
-		      syslog_options,
-		      syslog_facility,
-		      syslog_mask);
+                      self,
+                      syslog_ident,
+                      syslog_options,
+                      syslog_facility,
+                      syslog_mask);
 }
 
 /* Returns self, for backward compatibility.
@@ -420,6 +417,7 @@ static VALUE mSyslogMacros_included(VALUE mod, VALUE target)
  */
 void Init_syslog(void)
 {
+#undef rb_intern
     mSyslog = rb_define_module("Syslog");
 
     mSyslogConstants    = rb_define_module_under(mSyslog, "Constants");
@@ -578,6 +576,8 @@ void Init_syslog(void)
 
     /* Syslog macros */
 
+    rb_define_const(mSyslog, "VERSION", rb_str_new_cstr(SYSLOG_VERSION));
+
     rb_define_method(mSyslogMacros, "LOG_MASK", mSyslogMacros_LOG_MASK, 1);
     rb_define_method(mSyslogMacros, "LOG_UPTO", mSyslogMacros_LOG_UPTO, 1);
     rb_define_singleton_method(mSyslogMacros, "included", mSyslogMacros_included, 1);
