@@ -4,8 +4,8 @@ RSpec.describe "bundle install with groups" do
   describe "installing with no options" do
     before :each do
       install_gemfile <<-G
-        source "#{file_uri_for(gem_repo1)}"
-        gem "rack"
+        source "https://gem.repo1"
+        gem "myrack"
         group :emo do
           gem "activesupport", "2.3.5"
         end
@@ -14,7 +14,7 @@ RSpec.describe "bundle install with groups" do
     end
 
     it "installs gems in the default group" do
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
 
     it "installs gems in a group block into that group" do
@@ -40,7 +40,7 @@ RSpec.describe "bundle install with groups" do
     end
 
     it "sets up everything if Bundler.setup is used with no groups" do
-      output = run("require 'rack'; puts RACK")
+      output = run("require 'myrack'; puts MYRACK")
       expect(output).to eq("1.0.0")
 
       output = run("require 'activesupport'; puts ACTIVESUPPORT")
@@ -74,8 +74,8 @@ RSpec.describe "bundle install with groups" do
     describe "with gems assigned to a single group" do
       before :each do
         gemfile <<-G
-          source "#{file_uri_for(gem_repo1)}"
-          gem "rack"
+          source "https://gem.repo1"
+          gem "myrack"
           group :emo do
             gem "activesupport", "2.3.5"
           end
@@ -88,32 +88,32 @@ RSpec.describe "bundle install with groups" do
       it "installs gems in the default group" do
         bundle "config set --local without emo"
         bundle :install
-        expect(the_bundle).to include_gems "rack 1.0.0", :groups => [:default]
+        expect(the_bundle).to include_gems "myrack 1.0.0", groups: [:default]
       end
 
       it "respects global `without` configuration, but does not save it locally" do
         bundle "config set --global without emo"
         bundle :install
-        expect(the_bundle).to include_gems "rack 1.0.0", :groups => [:default]
+        expect(the_bundle).to include_gems "myrack 1.0.0", groups: [:default]
         bundle "config list"
         expect(out).not_to include("Set for your local app (#{bundled_app(".bundle/config")}): [:emo]")
         expect(out).to include("Set for the current user (#{home(".bundle/config")}): [:emo]")
       end
 
-      it "allows running application where groups where configured by a different user", :bundler => "< 3" do
+      it "allows running application where groups where configured by a different user", bundler: "< 3" do
         bundle "config set without emo"
         bundle :install
-        bundle "exec ruby -e 'puts 42'", :env => { "BUNDLE_USER_HOME" => tmp("new_home").to_s }
+        bundle "exec ruby -e 'puts 42'", env: { "BUNDLE_USER_HOME" => tmp("new_home").to_s }
         expect(out).to include("42")
       end
 
       it "does not install gems from the excluded group" do
         bundle "config set --local without emo"
         bundle :install
-        expect(the_bundle).not_to include_gems "activesupport 2.3.5", :groups => [:default]
+        expect(the_bundle).not_to include_gems "activesupport 2.3.5", groups: [:default]
       end
 
-      it "remembers previous exclusion with `--without`", :bundler => "< 3" do
+      it "remembers previous exclusion with `--without`", bundler: "< 3" do
         bundle "install --without emo"
         expect(the_bundle).not_to include_gems "activesupport 2.3.5"
         bundle :install
@@ -129,13 +129,13 @@ RSpec.describe "bundle install with groups" do
       it "allows Bundler.setup for specific groups" do
         bundle "config set --local without emo"
         bundle :install
-        run("require 'rack'; puts RACK", :default)
+        run("require 'myrack'; puts MYRACK", :default)
         expect(out).to eq("1.0.0")
       end
 
       it "does not effect the resolve" do
         gemfile <<-G
-          source "#{file_uri_for(gem_repo1)}"
+          source "https://gem.repo1"
           gem "activesupport"
           group :emo do
             gem "rails", "2.3.2"
@@ -144,7 +144,7 @@ RSpec.describe "bundle install with groups" do
 
         bundle "config set --local without emo"
         bundle :install
-        expect(the_bundle).to include_gems "activesupport 2.3.2", :groups => [:default]
+        expect(the_bundle).to include_gems "activesupport 2.3.2", groups: [:default]
       end
 
       it "still works when BUNDLE_WITHOUT is set" do
@@ -153,20 +153,20 @@ RSpec.describe "bundle install with groups" do
         bundle :install
         expect(out).not_to include("activesupport")
 
-        expect(the_bundle).to include_gems "rack 1.0.0", :groups => [:default]
-        expect(the_bundle).not_to include_gems "activesupport 2.3.5", :groups => [:default]
+        expect(the_bundle).to include_gems "myrack 1.0.0", groups: [:default]
+        expect(the_bundle).not_to include_gems "activesupport 2.3.5", groups: [:default]
 
         ENV["BUNDLE_WITHOUT"] = nil
       end
 
-      it "clears --without when passed an empty list", :bundler => "< 3" do
+      it "clears --without when passed an empty list", bundler: "< 3" do
         bundle "install --without emo"
 
         bundle "install --without ''"
         expect(the_bundle).to include_gems "activesupport 2.3.5"
       end
 
-      it "doesn't clear without when nothing is passed", :bundler => "< 3" do
+      it "doesn't clear without when nothing is passed", bundler: "< 3" do
         bundle "install --without emo"
 
         bundle :install
@@ -184,7 +184,7 @@ RSpec.describe "bundle install with groups" do
         expect(the_bundle).to include_gems "thin 1.0"
       end
 
-      it "installs gems from the previously requested group", :bundler => "< 3" do
+      it "installs gems from the previously requested group", bundler: "< 3" do
         bundle "install --with debugging"
         expect(the_bundle).to include_gems "thin 1.0"
         bundle :install
@@ -198,26 +198,26 @@ RSpec.describe "bundle install with groups" do
         ENV["BUNDLE_WITH"] = nil
       end
 
-      it "clears --with when passed an empty list", :bundler => "< 3" do
+      it "clears --with when passed an empty list", bundler: "< 3" do
         bundle "install --with debugging"
         bundle "install --with ''"
         expect(the_bundle).not_to include_gems "thin 1.0"
       end
 
-      it "removes groups from without when passed at --with", :bundler => "< 3" do
+      it "removes groups from without when passed at --with", bundler: "< 3" do
         bundle "config set --local without emo"
         bundle "install --with emo"
         expect(the_bundle).to include_gems "activesupport 2.3.5"
       end
 
-      it "removes groups from with when passed at --without", :bundler => "< 3" do
+      it "removes groups from with when passed at --without", bundler: "< 3" do
         bundle "config set --local with debugging"
-        bundle "install --without debugging", :raise_on_error => false
+        bundle "install --without debugging", raise_on_error: false
         expect(the_bundle).not_to include_gem "thin 1.0"
       end
 
-      it "errors out when passing a group to with and without via CLI flags", :bundler => "< 3" do
-        bundle "install --with emo debugging --without emo", :raise_on_error => false
+      it "errors out when passing a group to with and without via CLI flags", bundler: "< 3" do
+        bundle "install --with emo debugging --without emo", raise_on_error: false
         expect(last_command).to be_failure
         expect(err).to include("The offending groups are: emo")
       end
@@ -235,7 +235,7 @@ RSpec.describe "bundle install with groups" do
         expect(the_bundle).to include_gem "thin 1.0"
       end
 
-      it "can add and remove a group at the same time", :bundler => "< 3" do
+      it "can add and remove a group at the same time", bundler: "< 3" do
         bundle "install --with debugging --without emo"
         expect(the_bundle).to include_gems "thin 1.0"
         expect(the_bundle).not_to include_gems "activesupport 2.3.5"
@@ -257,8 +257,8 @@ RSpec.describe "bundle install with groups" do
     describe "with gems assigned to multiple groups" do
       before :each do
         gemfile <<-G
-          source "#{file_uri_for(gem_repo1)}"
-          gem "rack"
+          source "https://gem.repo1"
+          gem "myrack"
           group :emo, :lolercoaster do
             gem "activesupport", "2.3.5"
           end
@@ -268,20 +268,20 @@ RSpec.describe "bundle install with groups" do
       it "installs gems in the default group" do
         bundle "config set --local without emo lolercoaster"
         bundle :install
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
 
       it "installs the gem if any of its groups are installed" do
         bundle "config set --local without emo"
         bundle :install
-        expect(the_bundle).to include_gems "rack 1.0.0", "activesupport 2.3.5"
+        expect(the_bundle).to include_gems "myrack 1.0.0", "activesupport 2.3.5"
       end
 
       describe "with a gem defined multiple times in different groups" do
         before :each do
           gemfile <<-G
-            source "#{file_uri_for(gem_repo1)}"
-            gem "rack"
+            source "https://gem.repo1"
+            gem "myrack"
 
             group :emo do
               gem "activesupport", "2.3.5"
@@ -316,8 +316,8 @@ RSpec.describe "bundle install with groups" do
     describe "nesting groups" do
       before :each do
         gemfile <<-G
-          source "#{file_uri_for(gem_repo1)}"
-          gem "rack"
+          source "https://gem.repo1"
+          gem "myrack"
           group :emo do
             group :lolercoaster do
               gem "activesupport", "2.3.5"
@@ -329,13 +329,13 @@ RSpec.describe "bundle install with groups" do
       it "installs gems in the default group" do
         bundle "config set --local without emo lolercoaster"
         bundle :install
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
 
       it "installs the gem if any of its groups are installed" do
         bundle "config set --local without emo"
         bundle :install
-        expect(the_bundle).to include_gems "rack 1.0.0", "activesupport 2.3.5"
+        expect(the_bundle).to include_gems "myrack 1.0.0", "activesupport 2.3.5"
       end
     end
   end
@@ -343,16 +343,16 @@ RSpec.describe "bundle install with groups" do
   describe "when loading only the default group" do
     it "should not load all groups" do
       install_gemfile <<-G
-        source "#{file_uri_for(gem_repo1)}"
-        gem "rack"
+        source "https://gem.repo1"
+        gem "myrack"
         gem "activesupport", :groups => :development
       G
 
       ruby <<-R
-        require "#{entrypoint}"
+        require "bundler"
         Bundler.setup :default
         Bundler.require :default
-        puts RACK
+        puts MYRACK
         begin
           require "activesupport"
         rescue LoadError
@@ -369,34 +369,34 @@ RSpec.describe "bundle install with groups" do
     before(:each) do
       build_repo2
 
-      system_gems "rack-0.9.1"
+      system_gems "myrack-0.9.1"
 
-      bundle "config set --local without rack"
+      bundle "config set --local without myrack"
       install_gemfile <<-G
-        source "#{file_uri_for(gem_repo2)}"
-        gem "rack"
+        source "https://gem.repo2"
+        gem "myrack"
 
-        group :rack do
-          gem "rack_middleware"
+        group :myrack do
+          gem "myrack_middleware"
         end
       G
     end
 
     it "uses the correct versions even if --without was used on the original" do
-      expect(the_bundle).to include_gems "rack 0.9.1"
-      expect(the_bundle).not_to include_gems "rack_middleware 1.0"
+      expect(the_bundle).to include_gems "myrack 0.9.1"
+      expect(the_bundle).not_to include_gems "myrack_middleware 1.0"
       simulate_new_machine
 
       bundle :install
 
-      expect(the_bundle).to include_gems "rack 0.9.1"
-      expect(the_bundle).to include_gems "rack_middleware 1.0"
+      expect(the_bundle).to include_gems "myrack 0.9.1"
+      expect(the_bundle).to include_gems "myrack_middleware 1.0"
     end
 
     it "does not hit the remote a second time" do
       FileUtils.rm_rf gem_repo2
-      bundle "config set --local without rack"
-      bundle :install, :verbose => true
+      bundle "config set --local without myrack"
+      bundle :install, verbose: true
       expect(last_command.stdboth).not_to match(/fetching/i)
     end
   end
