@@ -27,7 +27,7 @@ describe "Dir#fileno" do
 
   if has_dir_fileno
     it "returns the file descriptor of the dir" do
-      @dir.fileno.should be_kind_of(Fixnum)
+      @dir.fileno.should be_kind_of(Integer)
     end
   else
     it "raises an error when not implemented on the platform" do
