@@ -749,68 +749,31 @@ describe "A method" do
       end
     end
 
-    ruby_version_is ""..."3.0" do
-      evaluate <<-ruby do
-          def m(a, b: 1) [a, b] end
-        ruby
+    evaluate <<-ruby do
+        def m(a, b: 1) [a, b] end
+      ruby
 
-        m(2).should == [2, 1]
-        m(1, b: 2).should == [1, 2]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [{"a" => 1, b: 2}, 1]
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(a, **) a end
-        ruby
-
-        m(1).should == 1
-        m(1, a: 2, b: 3).should == 1
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == {"a" => 1, b: 2}
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(a, **k) [a, k] end
-        ruby
-
-        m(1).should == [1, {}]
-        m(1, a: 2, b: 3).should == [1, {a: 2, b: 3}]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [{"a" => 1, b: 2}, {}]
-        end
-      end
+      m(2).should == [2, 1]
+      m(1, b: 2).should == [1, 2]
+      -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
     end
 
-    ruby_version_is "3.0" do
-      evaluate <<-ruby do
-          def m(a, b: 1) [a, b] end
-        ruby
+    evaluate <<-ruby do
+        def m(a, **) a end
+      ruby
 
-        m(2).should == [2, 1]
-        m(1, b: 2).should == [1, 2]
-        -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
-      end
+      m(1).should == 1
+      m(1, a: 2, b: 3).should == 1
+      -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
+    end
 
-      evaluate <<-ruby do
-          def m(a, **) a end
-        ruby
+    evaluate <<-ruby do
+        def m(a, **k) [a, k] end
+      ruby
 
-        m(1).should == 1
-        m(1, a: 2, b: 3).should == 1
-        -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
-      end
-
-      evaluate <<-ruby do
-          def m(a, **k) [a, k] end
-        ruby
-
-        m(1).should == [1, {}]
-        m(1, a: 2, b: 3).should == [1, {a: 2, b: 3}]
-        -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
-      end
+      m(1).should == [1, {}]
+      m(1, a: 2, b: 3).should == [1, {a: 2, b: 3}]
+      -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
     end
 
     evaluate <<-ruby do
@@ -921,50 +884,23 @@ describe "A method" do
       result.should == [[1, 2, 3], 4, [5, 6], 7, [], 8]
     end
 
-    ruby_version_is ""..."3.0" do
-      evaluate <<-ruby do
-          def m(a=1, b:) [a, b] end
-        ruby
+    evaluate <<-ruby do
+        def m(a=1, b:) [a, b] end
+      ruby
 
-        m(b: 2).should == [1, 2]
-        m(2, b: 1).should == [2, 1]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [{"a" => 1}, 2]
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(a=1, b: 2) [a, b] end
-        ruby
-
-        m().should == [1, 2]
-        m(2).should == [2, 2]
-        m(b: 3).should == [1, 3]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [{"a" => 1}, 2]
-        end
-      end
+      m(b: 2).should == [1, 2]
+      m(2, b: 1).should == [2, 1]
+      -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
     end
 
-    ruby_version_is "3.0" do
-      evaluate <<-ruby do
-          def m(a=1, b:) [a, b] end
-        ruby
+    evaluate <<-ruby do
+        def m(a=1, b: 2) [a, b] end
+      ruby
 
-        m(b: 2).should == [1, 2]
-        m(2, b: 1).should == [2, 1]
-        -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
-      end
-
-      evaluate <<-ruby do
-          def m(a=1, b: 2) [a, b] end
-        ruby
-
-        m().should == [1, 2]
-        m(2).should == [2, 2]
-        m(b: 3).should == [1, 3]
-        -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
-      end
+      m().should == [1, 2]
+      m(2).should == [2, 2]
+      m(b: 3).should == [1, 3]
+      -> { m("a" => 1, b: 2) }.should raise_error(ArgumentError)
     end
 
     evaluate <<-ruby do
@@ -1013,292 +949,6 @@ describe "A method" do
       m(1, 2, 3).should == [[1, 2], 3]
     end
 
-    ruby_version_is ""...'3.0' do
-      evaluate <<-ruby do
-          def m(*, a:) a end
-        ruby
-
-        m(a: 1).should == 1
-        m(1, 2, a: 3).should == 3
-        suppress_keyword_warning do
-          m("a" => 1, a: 2).should == 2
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*a, b:) [a, b] end
-        ruby
-
-        m(b: 1).should == [[], 1]
-        m(1, 2, b: 3).should == [[1, 2], 3]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [[{"a" => 1}], 2]
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*, a: 1) a end
-        ruby
-
-        m().should == 1
-        m(1, 2).should == 1
-        m(a: 2).should == 2
-        m(1, a: 2).should == 2
-        suppress_keyword_warning do
-          m("a" => 1, a: 2).should == 2
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*a, b: 1) [a, b] end
-        ruby
-
-        m().should == [[], 1]
-        m(1, 2, 3, b: 4).should == [[1, 2, 3], 4]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [[{"a" => 1}], 2]
-        end
-
-        a = mock("splat")
-        a.should_not_receive(:to_ary)
-        m(*a).should == [[a], 1]
-      end
-
-      evaluate <<-ruby do
-          def m(*a, **) a end
-        ruby
-
-        m().should == []
-        m(1, 2, 3, a: 4, b: 5).should == [1, 2, 3]
-        m("a" => 1, a: 1).should == []
-        m(1, **{a: 2}).should == [1]
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash)
-        -> { m(**h) }.should raise_error(TypeError)
-      end
-
-      evaluate <<-ruby do
-          def m(*, **k) k end
-        ruby
-
-        m().should == {}
-        m(1, 2, 3, a: 4, b: 5).should == {a: 4, b: 5}
-        m("a" => 1, a: 1).should == {"a" => 1, a: 1}
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash).and_return({a: 1})
-        suppress_warning do
-          m(h).should == {a: 1}
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(a = nil, **k) [a, k] end
-        ruby
-
-        m().should == [nil, {}]
-        m("a" => 1).should == [nil, {"a" => 1}]
-        m(a: 1).should == [nil, {a: 1}]
-        m("a" => 1, a: 1).should == [nil, {"a" => 1, a: 1}]
-        m({ "a" => 1 }, a: 1).should == [{"a" => 1}, {a: 1}]
-        suppress_warning do
-          m({a: 1}, {}).should == [{a: 1}, {}]
-
-          h = {"a" => 1, b: 2}
-          m(h).should == [{"a" => 1}, {b: 2}]
-          h.should == {"a" => 1, b: 2}
-
-          h = {"a" => 1}
-          m(h).first.should == h
-
-          h = {}
-          r = m(h)
-          r.first.should be_nil
-          r.last.should == {}
-
-          hh = {}
-          h = mock("keyword splat empty hash")
-          h.should_receive(:to_hash).and_return(hh)
-          r = m(h)
-          r.first.should be_nil
-          r.last.should == {}
-
-          h = mock("keyword splat")
-          h.should_receive(:to_hash).and_return({"a" => 1, a: 2})
-          m(h).should == [{"a" => 1}, {a: 2}]
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*a, **k) [a, k] end
-        ruby
-
-        m().should == [[], {}]
-        m(1).should == [[1], {}]
-        m(a: 1, b: 2).should == [[], {a: 1, b: 2}]
-        m(1, 2, 3, a: 2).should == [[1, 2, 3], {a: 2}]
-
-        m("a" => 1).should == [[], {"a" => 1}]
-        m(a: 1).should == [[], {a: 1}]
-        m("a" => 1, a: 1).should == [[], {"a" => 1, a: 1}]
-        m({ "a" => 1 }, a: 1).should == [[{"a" => 1}], {a: 1}]
-        suppress_warning do
-          m({a: 1}, {}).should == [[{a: 1}], {}]
-        end
-        m({a: 1}, {"a" => 1}).should == [[{a: 1}, {"a" => 1}], {}]
-
-        bo = BasicObject.new
-        def bo.to_a; [1, 2, 3]; end
-        def bo.to_hash; {:b => 2, :c => 3}; end
-
-        m(*bo, **bo).should == [[1, 2, 3], {:b => 2, :c => 3}]
-      end
-
-      evaluate <<-ruby do
-          def m(*, a:) a end
-        ruby
-
-        m(a: 1).should == 1
-        m(1, 2, a: 3).should == 3
-        suppress_keyword_warning do
-          m("a" => 1, a: 2).should == 2
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*a, b:) [a, b] end
-        ruby
-
-        m(b: 1).should == [[], 1]
-        m(1, 2, b: 3).should == [[1, 2], 3]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [[{"a" => 1}], 2]
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*, a: 1) a end
-        ruby
-
-        m().should == 1
-        m(1, 2).should == 1
-        m(a: 2).should == 2
-        m(1, a: 2).should == 2
-        suppress_keyword_warning do
-          m("a" => 1, a: 2).should == 2
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(*a, b: 1) [a, b] end
-        ruby
-
-        m().should == [[], 1]
-        m(1, 2, 3, b: 4).should == [[1, 2, 3], 4]
-        suppress_keyword_warning do
-          m("a" => 1, b: 2).should == [[{"a" => 1}], 2]
-        end
-
-        a = mock("splat")
-        a.should_not_receive(:to_ary)
-        m(*a).should == [[a], 1]
-      end
-
-      evaluate <<-ruby do
-          def m(*a, **) a end
-        ruby
-
-        m().should == []
-        m(1, 2, 3, a: 4, b: 5).should == [1, 2, 3]
-        m("a" => 1, a: 1).should == []
-        m(1, **{a: 2}).should == [1]
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash)
-        -> { m(**h) }.should raise_error(TypeError)
-      end
-
-      evaluate <<-ruby do
-          def m(*, **k) k end
-        ruby
-
-        m().should == {}
-        m(1, 2, 3, a: 4, b: 5).should == {a: 4, b: 5}
-        m("a" => 1, a: 1).should == {"a" => 1, a: 1}
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash).and_return({a: 1})
-        suppress_keyword_warning do
-          m(h).should == {a: 1}
-        end
-      end
-
-      evaluate <<-ruby do
-          def m(a = nil, **k) [a, k] end
-        ruby
-
-        m().should == [nil, {}]
-        m("a" => 1).should == [nil, {"a" => 1}]
-        m(a: 1).should == [nil, {a: 1}]
-        m("a" => 1, a: 1).should == [nil, {"a" => 1, a: 1}]
-        m({ "a" => 1 }, a: 1).should == [{"a" => 1}, {a: 1}]
-        suppress_keyword_warning do
-          m({a: 1}, {}).should == [{a: 1}, {}]
-        end
-
-        h = {"a" => 1, b: 2}
-        suppress_keyword_warning do
-          m(h).should == [{"a" => 1}, {b: 2}]
-        end
-        h.should == {"a" => 1, b: 2}
-
-        h = {"a" => 1}
-        m(h).first.should == h
-
-        h = {}
-        suppress_keyword_warning do
-          m(h).should == [nil, {}]
-        end
-
-        hh = {}
-        h = mock("keyword splat empty hash")
-        h.should_receive(:to_hash).and_return({a: 1})
-        suppress_keyword_warning do
-          m(h).should == [nil, {a: 1}]
-        end
-
-        h = mock("keyword splat")
-        h.should_receive(:to_hash).and_return({"a" => 1})
-        m(h).should == [h, {}]
-      end
-
-      evaluate <<-ruby do
-          def m(*a, **k) [a, k] end
-        ruby
-
-        m().should == [[], {}]
-        m(1).should == [[1], {}]
-        m(a: 1, b: 2).should == [[], {a: 1, b: 2}]
-        m(1, 2, 3, a: 2).should == [[1, 2, 3], {a: 2}]
-
-        m("a" => 1).should == [[], {"a" => 1}]
-        m(a: 1).should == [[], {a: 1}]
-        m("a" => 1, a: 1).should == [[], {"a" => 1, a: 1}]
-        m({ "a" => 1 }, a: 1).should == [[{"a" => 1}], {a: 1}]
-        suppress_keyword_warning do
-          m({a: 1}, {}).should == [[{a: 1}], {}]
-        end
-        m({a: 1}, {"a" => 1}).should == [[{a: 1}, {"a" => 1}], {}]
-
-        bo = BasicObject.new
-        def bo.to_a; [1, 2, 3]; end
-        def bo.to_hash; {:b => 2, :c => 3}; end
-
-        m(*bo, **bo).should == [[1, 2, 3], {:b => 2, :c => 3}]
-      end
-    end
-
     evaluate <<-ruby do
         def m(*, &b) b end
       ruby
@@ -1457,47 +1107,25 @@ describe "A method" do
       m({a: 1}).should == {a: 1}
       m({"a" => 1}).should == {"a" => 1}
 
-      -> { m(a: 1) }.should raise_error(ArgumentError)
-      -> { m(**{a: 1}) }.should raise_error(ArgumentError)
-      -> { m("a" => 1) }.should raise_error(ArgumentError)
+      -> { m(a: 1) }.should raise_error(ArgumentError, 'no keywords accepted')
+      -> { m(**{a: 1}) }.should raise_error(ArgumentError, 'no keywords accepted')
+      -> { m("a" => 1) }.should raise_error(ArgumentError, 'no keywords accepted')
     end
 
-    ruby_version_is ''...'3.0' do
-      evaluate <<-ruby do
-          def m(a, b = nil, c = nil, d, e: nil, **f)
-            [a, b, c, d, e, f]
-          end
-        ruby
-
-        result = m(1, 2)
-        result.should == [1, nil, nil, 2, nil, {}]
-
-        suppress_warning do
-          result = m(1, 2, {foo: :bar})
-          result.should == [1, nil, nil, 2, nil, {foo: :bar}]
+    evaluate <<-ruby do
+        def m(a, b = nil, c = nil, d, e: nil, **f)
+          [a, b, c, d, e, f]
         end
+      ruby
 
-        result = m(1, {foo: :bar})
-        result.should == [1, nil, nil, {foo: :bar}, nil, {}]
-      end
-    end
+      result = m(1, 2)
+      result.should == [1, nil, nil, 2, nil, {}]
 
-    ruby_version_is '3.0' do
-      evaluate <<-ruby do
-          def m(a, b = nil, c = nil, d, e: nil, **f)
-            [a, b, c, d, e, f]
-          end
-        ruby
+      result = m(1, 2, {foo: :bar})
+      result.should == [1, 2, nil, {foo: :bar}, nil, {}]
 
-        result = m(1, 2)
-        result.should == [1, nil, nil, 2, nil, {}]
-
-        result = m(1, 2, {foo: :bar})
-        result.should == [1, 2, nil, {foo: :bar}, nil, {}]
-
-        result = m(1, {foo: :bar})
-        result.should == [1, nil, nil, {foo: :bar}, nil, {}]
-      end
+      result = m(1, {foo: :bar})
+      result.should == [1, nil, nil, {foo: :bar}, nil, {}]
     end
   end
 
@@ -1511,60 +1139,27 @@ describe "A method" do
     end
   end
 
-  ruby_version_is ''...'3.0' do
-    context 'when passing an empty keyword splat to a method that does not accept keywords' do
-      evaluate <<-ruby do
-          def m(a); a; end
-        ruby
-        h = {}
+  context 'when passing an empty keyword splat to a method that does not accept keywords' do
+    evaluate <<-ruby do
+        def m(a); a; end
+      ruby
+      h = {}
 
-        -> do
-          m(**h).should == {}
-        end.should complain(/warning: Passing the keyword argument as the last hash parameter is deprecated/)
-      end
+      -> do
+        m(**h).should == {}
+      end.should raise_error(ArgumentError)
     end
   end
 
-  ruby_version_is ''...'3.0' do
-    context "assigns keyword arguments from a passed Hash without modifying it" do
-      evaluate <<-ruby do
-          def m(a: nil); a; end
-        ruby
+  context "raises ArgumentError if passing hash as keyword arguments" do
+    evaluate <<-ruby do
+        def m(a: nil); a; end
+      ruby
 
-        options = {a: 1}.freeze
-        -> do
-          suppress_warning do
-            m(options).should == 1
-          end
-        end.should_not raise_error
-        options.should == {a: 1}
-      end
-    end
-  end
-
-  ruby_version_is '3.0' do
-    context 'when passing an empty keyword splat to a method that does not accept keywords' do
-      evaluate <<-ruby do
-          def m(a); a; end
-        ruby
-        h = {}
-
-        -> do
-          m(**h).should == {}
-        end.should raise_error(ArgumentError)
-      end
-    end
-
-    context "raises ArgumentError if passing hash as keyword arguments" do
-      evaluate <<-ruby do
-          def m(a: nil); a; end
-        ruby
-
-        options = {a: 1}.freeze
-        -> do
-          m(options)
-        end.should raise_error(ArgumentError)
-      end
+      options = {a: 1}.freeze
+      -> do
+        m(options)
+      end.should raise_error(ArgumentError)
     end
   end
 
@@ -1598,14 +1193,42 @@ describe "A method call with a space between method name and parentheses" do
     end
   end
 
-  context "when a single argument provided" do
-    it "assigns it" do
+  context "when a single argument is provided" do
+    it "assigns a simple expression" do
+      args = m (1)
+      args.should == [1]
+    end
+
+    it "assigns an expression consisting of multiple statements" do
+      args = m ((0; 1))
+      args.should == [1]
+    end
+
+    it "assigns one single statement, without the need of parentheses" do
       args = m (1 == 1 ? true : false)
       args.should == [true]
     end
+
+    ruby_version_is "3.3" do
+      it "supports multiple statements" do
+        eval("m (1; 2)").should == [2]
+      end
+    end
   end
 
-  context "when 2+ arguments provided" do
+  context "when multiple arguments are provided" do
+    it "assigns simple expressions" do
+      args = m (1), (2)
+      args.should == [1, 2]
+    end
+
+    it "assigns expressions consisting of multiple statements" do
+      args = m ((0; 1)), ((2; 3))
+      args.should == [1, 3]
+    end
+  end
+
+  context "when the argument looks like an argument list" do
     it "raises a syntax error" do
       -> {
         eval("m (1, 2)")
@@ -1670,105 +1293,107 @@ describe "An array-dereference method ([])" do
   end
 end
 
-ruby_version_is "3.0" do
-  describe "An endless method definition" do
-    context "without arguments" do
+describe "An endless method definition" do
+  context "without arguments" do
+    evaluate <<-ruby do
+        def m() = 42
+      ruby
+
+      m.should == 42
+    end
+
+    context "without parenthesis" do
       evaluate <<-ruby do
-          def m() = 42
-        ruby
+        def m = 42
+      ruby
 
         m.should == 42
       end
-
-      context "without parenthesis" do
-        evaluate <<-ruby do
-          def m = 42
-        ruby
-
-          m.should == 42
-        end
-      end
-    end
-
-    context "with arguments" do
-      evaluate <<-ruby do
-          def m(a, b) = a + b
-        ruby
-
-        m(1, 4).should == 5
-      end
-    end
-
-    context "with multiline body" do
-      evaluate <<-ruby do
-          def m(n) =
-            if n > 2
-              m(n - 2) + m(n - 1)
-            else
-              1
-            end
-        ruby
-
-        m(6).should == 8
-      end
-    end
-
-    context "with args forwarding" do
-      evaluate <<-ruby do
-          def mm(word, num:)
-            word * num
-          end
-
-          def m(...) = mm(...) + mm(...)
-        ruby
-
-        m("meow", num: 2).should == "meow" * 4
-      end
-    end
-
-    ruby_version_is ""..."3.0" do
-      context "inside 'endless' method definitions" do
-        it "does not allow method calls without parenthesis" do
-          -> {
-            eval("def greet(person) = 'Hi, '.concat person")
-          }.should raise_error(SyntaxError)
-        end
-      end
     end
   end
 
-  describe "Keyword arguments are now separated from positional arguments" do
-    context "when the method has only positional parameters" do
-      it "treats incoming keyword arguments as positional for compatibility" do
-        def foo(a, b, c, hsh)
-          hsh[:key]
+  context "with arguments" do
+    evaluate <<-ruby do
+        def m(a, b) = a + b
+      ruby
+
+      m(1, 4).should == 5
+    end
+  end
+
+  context "with multiline body" do
+    evaluate <<-ruby do
+        def m(n) =
+          if n > 2
+            m(n - 2) + m(n - 1)
+          else
+            1
+          end
+      ruby
+
+      m(6).should == 8
+    end
+  end
+
+  context "with args forwarding" do
+    evaluate <<-ruby do
+        def mm(word, num:)
+          word * num
         end
 
-        foo(1, 2, 3, key: 42).should == 42
+        def m(...) = mm(...) + mm(...)
+      ruby
+
+      m("meow", num: 2).should == "meow" * 4
+    end
+  end
+end
+
+describe "Keyword arguments are now separated from positional arguments" do
+  context "when the method has only positional parameters" do
+    it "treats incoming keyword arguments as positional for compatibility" do
+      def foo(a, b, c, hsh)
+        hsh[:key]
       end
+
+      foo(1, 2, 3, key: 42).should == 42
+    end
+  end
+
+  context "when the method takes a ** parameter" do
+    it "captures the passed literal keyword arguments" do
+      def foo(a, b, c, **hsh)
+        hsh[:key]
+      end
+
+      foo(1, 2, 3, key: 42).should == 42
     end
 
-    context "when the method takes a ** parameter" do
-      it "captures the passed literal keyword arguments" do
-        def foo(a, b, c, **hsh)
-          hsh[:key]
-        end
-
-        foo(1, 2, 3, key: 42).should == 42
+    it "captures the passed ** keyword arguments" do
+      def foo(a, b, c, **hsh)
+        hsh[:key]
       end
 
-      it "captures the passed ** keyword arguments" do
-        def foo(a, b, c, **hsh)
-          hsh[:key]
-        end
+      h = { key: 42 }
+      foo(1, 2, 3, **h).should == 42
+    end
 
-        h = { key: 42 }
-        foo(1, 2, 3, **h).should == 42
+    it "does not convert a positional Hash to keyword arguments" do
+      def foo(a, b, c, **hsh)
+        hsh[:key]
       end
 
-      it "does not convert a positional Hash to keyword arguments" do
-        def foo(a, b, c, **hsh)
-          hsh[:key]
+      -> {
+        foo(1, 2, 3, { key: 42 })
+      }.should raise_error(ArgumentError, 'wrong number of arguments (given 4, expected 3)')
+    end
+  end
+
+  context "when the method takes a key: parameter" do
+    context "when it's called with a positional Hash and no **" do
+      it "raises ArgumentError" do
+        def foo(a, b, c, key: 1)
+          key
         end
 
         -> {
@@ -1777,28 +1402,14 @@ ruby_version_is "3.0" do
       end
     end
 
-    context "when the method takes a key: parameter" do
-      context "when it's called with a positional Hash and no **" do
-        it "raises ArgumentError" do
-          def foo(a, b, c, key: 1)
-            key
-          end
-
-          -> {
-            foo(1, 2, 3, { key: 42 })
-          }.should raise_error(ArgumentError, 'wrong number of arguments (given 4, expected 3)')
+    context "when it's called with **" do
+      it "captures the passed keyword arguments" do
+        def foo(a, b, c, key: 1)
+          key
         end
-      end
 
-      context "when it's called with **" do
-        it "captures the passed keyword arguments" do
-          def foo(a, b, c, key: 1)
-            key
-          end
-
-          h = { key: 42 }
-          foo(1, 2, 3, **h).should == 42
-        end
+        h = { key: 42 }
+        foo(1, 2, 3, **h).should == 42
       end
     end
   end
