@@ -9,7 +9,6 @@
 
 **********************************************************************/
 
-#include "gc.h"
 #include "internal.h"
 #include "internal/error.h"
 #include "internal/gc.h"
@@ -23,7 +22,6 @@
 #include "symbol.h"
 #include "vm_sync.h"
 #include "builtin.h"
-#include "ruby/internal/attr/nonstring.h"
 
 #if defined(USE_SYMBOL_GC) && !(USE_SYMBOL_GC+0)
 # undef USE_SYMBOL_GC
@@ -32,7 +30,11 @@
 # undef USE_SYMBOL_GC
 # define USE_SYMBOL_GC 1
 #endif
-#ifndef SYMBOL_DEBUG
+#if defined(SYMBOL_DEBUG) && (SYMBOL_DEBUG+0)
+# undef SYMBOL_DEBUG
+# define SYMBOL_DEBUG 1
+#else
+# undef SYMBOL_DEBUG
 # define SYMBOL_DEBUG 0
 #endif
 #ifndef CHECK_ID_SERIAL
@@ -153,7 +155,7 @@ rb_id_attrset(ID id)
 
     /* make new symbol and ID */
     if (!(str = lookup_id_str(id))) {
-        RBIMPL_ATTR_NONSTRING() static const char id_types[][8] = {
+        static const char id_types[][8] = {
             "local",
             "instance",
             "invalid",
@@ -458,8 +460,7 @@ get_id_serial_entry(rb_id_serial_t num, ID id, const enum id_entry_type t)
                 if (NIL_P(result)) {
                     result = 0;
                 }
-                else {
-#if CHECK_ID_SERIAL
+                else if (CHECK_ID_SERIAL) {
                     if (id) {
                         VALUE sym = result;
                         if (t != ID_ENTRY_SYM)
@@ -471,7 +472,6 @@ get_id_serial_entry(rb_id_serial_t num, ID id, const enum id_entry_type t)
                             if (RSYMBOL(sym)->id != id) result = 0;
                         }
                     }
-#endif
                 }
             }
         }
@@ -506,7 +506,6 @@ rb_id_serial_to_id(rb_id_serial_t num)
     }
 }
 
-#if SYMBOL_DEBUG
 static int
 register_sym_update_callback(st_data_t *key, st_data_t *value, st_data_t arg, int existing)
 {
@@ -517,19 +516,29 @@ register_sym_update_callback(st_data_t *key, st_data_t *value, st_data_t arg, in
     *value = arg;
     return ST_CONTINUE;
 }
-#endif
 
 static void
 register_sym(rb_symbols_t *symbols, VALUE str, VALUE sym)
 {
     ASSERT_vm_locking();
 
-#if SYMBOL_DEBUG
-    st_update(symbols->str_sym, (st_data_t)str,
-              register_sym_update_callback, (st_data_t)sym);
-#else
-    st_add_direct(symbols->str_sym, (st_data_t)str, (st_data_t)sym);
-#endif
+    if (SYMBOL_DEBUG) {
+        st_update(symbols->str_sym, (st_data_t)str,
+                  register_sym_update_callback, (st_data_t)sym);
+    }
+    else {
+        st_add_direct(symbols->str_sym, (st_data_t)str, (st_data_t)sym);
+    }
+}
+
+void
+rb_free_static_symid_str(void)
+{
+    GLOBAL_SYMBOLS_ENTER(symbols)
+    {
+        st_free_table(symbols->str_sym);
+    }
+    GLOBAL_SYMBOLS_LEAVE();
 }
 
 static void
