@@ -1,5 +1,5 @@
 # frozen_string_literal: true
-require 'rdoc/test_case'
+require_relative 'helper'
 begin
   require 'rake'
 rescue LoadError
@@ -20,19 +20,19 @@ class TestRDocTask < RDoc::TestCase
   end
 
   def test_inline_source
-    _, err = verbose_capture_io do
+    _, err = verbose_capture_output do
       assert @t.inline_source
     end
 
     assert_equal "RDoc::Task#inline_source is deprecated\n", err
 
-    _, err = verbose_capture_io do
+    _, err = verbose_capture_output do
       @t.inline_source = false
     end
 
     assert_equal "RDoc::Task#inline_source is deprecated\n", err
 
-    capture_io do
+    capture_output do
       assert @t.inline_source
     end
   end
@@ -50,6 +50,7 @@ class TestRDocTask < RDoc::TestCase
     assert Rake::Task[:rdoc]
     assert Rake::Task[:clobber_rdoc]
     assert Rake::Task[:rerdoc]
+    assert Rake::Task[:"rdoc:coverage"]
     assert_equal ["html/created.rid"], Rake::Task[:rdoc].prerequisites
   end
 
@@ -58,6 +59,7 @@ class TestRDocTask < RDoc::TestCase
     assert Rake::Task[:rdoc_dev]
     assert Rake::Task[:clobber_rdoc_dev]
     assert Rake::Task[:rerdoc_dev]
+    assert Rake::Task[:"rdoc_dev:coverage"]
     assert_equal :rdoc_dev, rd.name
   end
 
@@ -69,8 +71,8 @@ class TestRDocTask < RDoc::TestCase
       rd.options << "--all"
     end
 
-    assert rdoc_task.title, "Test Tasks Option Parser"
-    assert rdoc_task.main, "README.md"
+    assert_equal "Test Tasks Option Parser", rdoc_task.title
+    assert_equal "README.md", rdoc_task.main
     assert rdoc_task.rdoc_files.include?("README.md")
     assert rdoc_task.options.include?("--all")
 
@@ -110,11 +112,16 @@ class TestRDocTask < RDoc::TestCase
     assert_equal 'Rebuild RDoc HTML files', @t.rerdoc_task_description
   end
 
+  def test_coverage_task_description
+    assert_equal 'Print RDoc coverage report', @t.coverage_task_description
+  end
+
   def test_tasks_creation_with_custom_name_string
     rd = RDoc::Task.new("rdoc_dev")
     assert Rake::Task[:rdoc_dev]
     assert Rake::Task[:clobber_rdoc_dev]
     assert Rake::Task[:rerdoc_dev]
+    assert Rake::Task[:"rdoc_dev:coverage"]
     assert_equal "rdoc_dev", rd.name
   end
 
@@ -131,7 +138,8 @@ class TestRDocTask < RDoc::TestCase
     assert Rake::Task[:"rdoc"]
     assert Rake::Task[:"rdoc:clean"]
     assert Rake::Task[:"rdoc:force"]
-    assert_raises(RuntimeError) { Rake::Task[:clobber_rdoc] }
+    assert Rake::Task[:"rdoc:coverage"]
+    assert_raise(RuntimeError) { Rake::Task[:clobber_rdoc] }
     assert_equal options, rd.name
   end
 
@@ -140,10 +148,11 @@ class TestRDocTask < RDoc::TestCase
     assert Rake::Task[:rdoc]
     assert Rake::Task[:"rdoc:clean"]
     assert Rake::Task[:rerdoc]
+    assert Rake::Task[:"rdoc:coverage"]
   end
 
   def test_tasks_creation_with_custom_name_hash_raises_exception_if_invalid_option_given
-    assert_raises(ArgumentError) do
+    assert_raise(ArgumentError) do
       RDoc::Task.new(:foo => "bar")
     end
 
@@ -171,4 +180,3 @@ class TestRDocTask < RDoc::TestCase
   end
 
 end if defined?(Rake::Task)
-
